#include <numeric>
#include <vector>
#include "testlib.h"
using namespace std;
using ll = long long int;
using pii = pair<int, int>;

struct IncorrectInfo {
    int pos;
    int leftTaller;
    int rightTaller;
};

bool isCorrect(const vector<int>& vec, IncorrectInfo& info) {
    int n = vec.size();

    auto left = vec;
    auto right = vec;
    vector<int> leftIndex(n);
    vector<int> rightIndex(n);

    leftIndex[0] = 0;
    for (int i = 1; i < n; ++i) {
        left[i] = max(left[i], left[i - 1]);
        if (left[i] == left[i - 1]) {
            leftIndex[i] = leftIndex[i - 1];
        }
    }

    rightIndex[n - 1] = n - 1;
    for (int i = n - 2; i >= 0; --i) {
        right[i] = max(right[i], right[i + 1]);
        if (right[i] == right[i + 1]) {
            rightIndex[i] = rightIndex[i + 1];
        }
    }

    for (int i = 1; i < n - 1; ++i) {
        if (vec[i] < left[i - 1] && vec[i] < right[i + 1]) {
            info.pos = i;
            info.leftTaller = leftIndex[i - 1];
            info.rightTaller = rightIndex[i + 1];
            return false;
        }
    }

    return true;
}

vector<int> readAns(InStream& stream, int n, const vector<int>& bounds) {
    auto ans = stream.readInts(n, 1, 1e9, "m");
    for (int i = 0; i < n; i++) {
        if (ans[i] > bounds[i]) {
            stream.quitf(_wa, "The %d-th height is more that the bound (height: %d, bound: %d)", i, ans[i], bounds[i]);
        }
    }

    IncorrectInfo info;
    if (!isCorrect(ans, info)) {
        stream.quitf(_wa, "The %d-th height has taller left (%d-th) and right (%d-th) neighbors", info.pos, info.leftTaller, info.rightTaller);
    }

    return ans;
}

int main(int argc, char* argv[]) {
    registerTestlibCmd(argc, argv);

    int n = inf.readInt(1, 5e5, "n");
    auto bounds = inf.readInts(n, 1, 1e9, "m");

    auto pans = readAns(ouf, n, bounds);
    auto jans = readAns(ans, n, bounds);

    ll psum = accumulate(pans.begin(), pans.end(), 0LL);
    ll jsum = accumulate(jans.begin(), jans.end(), 0LL);

    if (psum > jsum) {
        quitf(_fail, "A participant found a better answer!!! Jury answer is %lld, participant answer is %lld", jsum, psum);
    }
    if (psum < jsum) {
        quitf(_wa, "Jury found a better answer %lld, participant answer is %lld", jsum, psum);
    }
    quitf(_ok, "The answer is correct");
}
