#include "testlib.h"

#include <algorithm>
#include <cassert>
#include <iostream>
#include <map>
#include <numeric>
#include <queue>
#include <set>
#include <string>
#include <vector>

using namespace std;
using ll = long long int;

// Абсолютный рандом
vector<int> gen_random(int size) {
    vector<int> res(size);
    iota(res.begin(), res.end(), 0);
    shuffle(res.begin(), res.end());
    return res;
}

// Генерирует гору (увеличивается, затем уменьшается)
vector<int> gen_small_at_bounds(int size) {
    vector<int> res(size);
    int l = 0, r = size - 1;
    for (int i = 0; i < size; ++i) {
        if (rnd.next(2)) {
            res[l++] = i;
        } else {
            res[r--] = i;
        }
    }
    return res;
}

// 1, n, 2, n-1, ..., n/2+1, n/2
vector<int> gen_two_sorted_and_merged(int size) {
    vector<int> res(size);
    iota(res.begin(), res.end(), 0);

    vector<int> indices;
    for (int i = 1; i < size; i += 2) {
        indices.push_back(i);
    }
    for (int i = 0; i < indices.size() - 1 - i; ++i) {
        swap(res[indices[i]], res[indices[indices.size() - 1 - i]]);
    }

    return res;
}

// Генерируем n разных чисел до границы
vector<int> gen_numbers(int size, int bound) {
    while (true) {
        vector<int> res(size);
        for (auto& it : res) {
            it = rnd.next(bound);
        }
        sort(res.begin(), res.end());
        int add = 0;
        for (int i = 1; i < size; ++i) {
            res[i] += add;
            if (res[i] == res[i - 1]) {
                ++add;
                ++res[i];
            }
        }

        bool ok = true;
        for (auto& it : res) {
            if (it > bound) {
                ok = false;
                break;
            }
        }
        if (ok) {
            return res;
        }
    }
}

// Генерирует убывающую гору
vector<int> gen_lowering_mountain(int size) {
    vector<int> res(size);
    for (int i = 0; i < size; ++i) {
        res[size - 1 - i] = i;
    }
    return res;
}

// Генерирует возрастающую гору
vector<int> gen_uppering_mountain(int size) {
    vector<int> res(size);
    for (int i = 0; i < size; ++i) {
        res[i] = i;
    }
    return res;
}

int main(int argc, char* argv[])
{
    registerGen(argc, argv, 1);

    int n = atoi(argv[1]);
    int parts = atoi(argv[2]);
    int bound = atoi(argv[3]);
    char* part_code = argv[4];

    // where parts are divided?
    vector<int> separators;
    vector<int> sizes;
    vector<int> starts;
    int start = 0;
    for (int i = 0; i < parts - 1; ++i) {
        int pos = start + (n - start) / (parts - i);
        separators.push_back(pos);

        starts.push_back(start);
        sizes.push_back(pos - start);
        start = pos + 1;
    }
    starts.push_back(start);
    sizes.push_back(n - start);

    // set up result array
    vector<int> res(n);
    auto numbers = gen_numbers(n, bound);
    int num = 0;

    // set up separators
    shuffle(separators.begin(), separators.end());
    for (auto it : separators) {
        res[it] = numbers[num++];
    }

    // set up parts
    vector<int> active_parts(parts);
    vector<queue<int>> parts_indices(parts);

    iota(active_parts.begin(), active_parts.end(), 0);

    for (int i = 0; i < parts; ++i) {
        vector<int> indices;
        int code = part_code[i] - '0';
        int size = sizes[i];

        if (code == 0) {
            indices = gen_random(size);
        } else if (code == 1) {
            indices = gen_small_at_bounds(size);
        } else if (code == 2) {
            indices = gen_two_sorted_and_merged(size);
        } else if (code == 3) {
            indices = gen_lowering_mountain(size);
        } else if (code == 4) {
            indices = gen_uppering_mountain(size);
        }
        
        vector<int> mapper(size);
        int start = starts[i];
        for (int z = 0; z < size; ++z) {
            mapper[indices[z]] = z + start;
        }

        for (auto it : mapper) {
            parts_indices[i].push(it);
        }
    }

    while (num < numbers.size()) {
        // select number
        int ind = rnd.next(active_parts.size());
        int queue_num = active_parts[ind];
        auto& q = parts_indices[queue_num];
        res[q.front()] = numbers[num++];
        
        q.pop();
        if (q.empty()) {
            active_parts.erase(active_parts.begin() + ind);
        }
    }

    // print out
    println(n);
    println(res);
}
